#!/bin/sh

target_device=/dev/mtdblock5
bootloader_file=u-boot.bin
bootscript_file="boot.scr"
bootscript_file_16k="/tmp/boot.scr"

# we put the boostscript file in an zeroed 16k dump to allow a driect compare with the flash dump
dd if=/dev/zero of=${bootscript_file_16k} conv=notrunc bs=16k count=1
dd if=${bootscript_file} of=${bootscript_file_16k} bs=16k conv=notrunc


getBootloaderChecksum()
{
  echo $(dd if=${target_device} conv=notrunc bs=16k skip=1 count=14 2>/dev/null | md5sum | cut -d " " -f1)
}

getBootscriptChecksum()
{
  echo $(dd if=${target_device} conv=notrunc bs=16k skip=15 count=1 2>/dev/null | md5sum | cut -d " " -f1)
}

getBootloaderFileChecksum()
{
  echo $(md5sum ${bootloader_file} | cut -d " " -f1)
}

getBootScriptFileChecksum()
{
  echo $(md5sum ${bootscript_file_16k} | cut -d " " -f1)
}

writeBootloader()
{
    dd if=${bootloader_file} of=${target_device} conv=notrunc bs=16k seek=1
    echo $?
}

writeBootScript()
{
    dd if=/dev/zero of=${target_device} conv=notrunc bs=16k seek=15 count=1
    dd if=${bootscript_file_16k} of=${target_device} conv=notrunc bs=16k seek=15 count=1
    echo $?
}

updateError()
{
  echo "Set app led to panic mode..."
  devicestack --ipc-send=led:0:p*
  exit 1
}

if [ -e "$bootloader_file" ]; then
  echo "Check if bootloader needs an update."

  echo "Bootloader checksums:"
  echo "Installed: " $(getBootloaderChecksum)
  echo "File     : " $(getBootloaderFileChecksum)

  ret=1
  count=0
  while [ $ret -ne 0  ]; do
    count=$(expr $count + 1)
    if [ $count -gt 6 ]; then
      echo "Tried five times to write bootloader but failed!"
      updateError
    fi
  
    if [ "$(getBootloaderChecksum)" != "$(getBootloaderFileChecksum)" ]; then
      echo "Bootloader needs to be updated"
      if [ "$(writeBootloader)" != "0" ]; then
        echo "Error: could not write bootloader file!" 
        ret=1
      else 
        if [ "$(getBootloaderChecksum)" != "$(getBootloaderFileChecksum)" ]; then
          echo "Bootloader updated but checksum is not equal!"
          echo "Bootloader checksums:"
          echo "Installed: " $(getBootloaderChecksum)
          echo "File     : " $(getBootloaderFileChecksum)
          ret=1
        else 
          echo "Bootloader successfully installed."
          ret=0
        fi
      fi
    else
      echo "Bootloader allready installed."
      ret=0
    fi
  done
fi

if [ -e "$bootscript_file" ]; then
  echo "Check if boot script needs an update."

  echo "Boot script checksums:"
  echo "Installed: " $(getBootscriptChecksum)
  echo "File     : " $(getBootScriptFileChecksum)

  ret=1
  count=0
  while [ $ret -ne 0  ]; do
    count=$(expr $count + 1)
    if [ $count -gt 6 ]; then
      echo "Tried five times to write boot script but failed!"
      updateError
    fi
  
    if [ "$(getBootscriptChecksum)" != "$(getBootScriptFileChecksum)" ]; then
      echo "Boot script needs to be updated"
      if [ "$(writeBootScript)" != "0" ]; then
        echo "Error: could not write boot script file!" 
        ret=1
      else 
        if [ "$(getBootscriptChecksum)" != "$(getBootScriptFileChecksum)" ]; then
          echo "Boot script updated but checksum is not equal!"
          echo "Boot script checksums:"
          echo "Installed: " $(getBootscriptChecksum)
          echo "File     : " $(getBootScriptFileChecksum)
          ret=1
        else 
          echo "Boot script successfully installed."
          ret=0
        fi
      fi
    else
      echo "Bootloader allready installed."
      ret=0
    fi
  done
fi

exit 0
